function ARKINV:RulesWindowClose()
	ARKINV_Rules:Hide()
end

function ARKINV:RulesTitleSet()

	-- set the title
	obj = getglobal( "ARKINV_RulesTitleWho" )

	t = ARKINV_L["MOD_RULES"] .. ": " .. ARKINV_G.PROGRAM_NAME .. " v" .. format( "%.2f", ARKINV_G.PROGRAM_VERSION )

	if ARKINV_G.PROGRAM_BETA then
		t = t .. " " .. RED_FONT_COLOR_CODE .. "(beta " .. ARKINV_G.PROGRAM_BETA .. ")"
	end
	
	obj:SetText( t )
	
end

function ARKINV:SetEditBoxLabel( arg1 )
	getglobal( this:GetName() .. "Label" ):SetText( arg1 )
end

function ARKINV:UiTabToNext()
	
	local k = { this:GetParent():GetChildren() }
	local i1 = this:GetID()
	
	--ARKINV:Print( "Tab From [" .. this:GetName() .. "] id=[" .. i1 .. "]" )
	
	if not i1 or i1 == 0 then
		ARKINV:Print( "OOPS: Attempted to tab from a widget with no ID [" .. this:GetName() .. "]" )
		return
	end
	
	if not k then
		ARKINV:Print( "OOPS: Parent [" .. this:GetParent():GetName() .. "] has no children?" )
	end

	local i2 = 999999
	
	-- scan through all children to find the next highest id
	local x
	for _, w in ipairs( k ) do
		--ARKINV:Print( "this=" .. w:GetName() .. ", id=" .. w:GetID() .. ", visible=" .. w:IsVisible() )
		if w:IsVisible() and w:GetID() > i1 and w:GetID() < i2 then
			x = w
			i2 = x:GetID()
		end
	end

	-- if found then focus and exit
	if x then
		--ARKINV:Print( "Tab To [" .. x:GetName() .. "] id=[" .. i2 .. "]" )
		x:SetFocus()
		return
	end

	-- if not found then we're at the end or there's only a single entry so look for the lowest id
	for _, w in ipairs( k ) do
		if w:IsVisible() and w:GetID() > 0 then
			--ARKINV:Print( "Tab To [" .. w:GetName() .. "] id=[" .. w:GetID() .. "]" )
			w:SetFocus()
			return
		end
	end
	
	-- should never get here
	ARKINV:Print( "OOPS: We shouldn't be here @ ARKINV:UiTabToNext()" )
	
end

function ARKINV:RuleStructureSort()

end

function ARKINV:RuleStructureRowItem( f )

	if not f then f = this:GetName() end
	
	local x
	
	-- id
	x = getglobal( f .. "C1" )
	x:ClearAllPoints()
	x:SetWidth( 50 )
	x:SetPoint( "RIGHT", 0, 0 )
	x:SetPoint( "TOP", 0, 0 )
	x:SetPoint( "BOTTOM", 0, 0 )
	x:SetTextColor( 1, 1, 1, 1 )
	x:SetJustifyH( "LEFT", 0, 0 )
	x:Show()

	-- description
	x = getglobal( f .. "C3" )
	x:ClearAllPoints()
	x:SetWidth( 60 )
	x:SetPoint( "RIGHT", f .. "C2", "LEFT", -5, 0 )
	x:SetPoint( "TOP", 0, 0 )
	x:SetPoint( "BOTTOM", 0, 0 )
	x:SetTextColor( 1, 1, 1, 1 )
	x:SetJustifyH( "RIGHT", 0, 0 )
	x:Show()

	-- formula
	x = getglobal( f .. "C1" )
	x:ClearAllPoints()
	x:SetPoint( "LEFT", f .. "T1", "RIGHT", 5, 0 )
	x:SetPoint( "RIGHT", f .. "C3", "LEFT", -5, 0 )
	x:SetPoint( "TOP", 0, 0 )
	x:SetPoint( "BOTTOM", 0, 0 )
	x:SetJustifyH( "LEFT", 0, 0 )
	x:Show()

end

function ARKINV:RuleStructureTable( f )

	if not f then f = this:GetName() end

	local p = getglobal( f )
	if not p then
		ARKINV:Print( "Invalid value at ARKINV:RuleStructureTable( [" .. ARKINV:MakeNullStringEmpty( f ) .. "] )" )
		return
	end
	
	local maxrows = tonumber( getglobal( f .. "MaxRows" ):GetText() )
	local rows = maxrows
	local height = 30
	
	if rows > maxrows then rows = maxrows end
	getglobal( f .. "NumRows" ):SetText( rows )
	
	if height == 0 then
		height = tonumber( getglobal( f .. "RowHeight" ):GetText() )
	end
	getglobal( f .. "RowHeight" ):SetText( height )

	-- stretch scrollbar to bottom row
	getglobal( f .. "Scroll" ):SetPoint( "BOTTOM", f .. "Row" .. rows, "BOTTOM", 0, 0 )

	-- set frame height to correct size
	getglobal( f ):SetHeight( height * rows + 20 )

end

function ARKINV:RuleTableClick( f )

	if not f then f = this:GetName() end
	
	ARKINV:Print( "RuleTableClick( " .. f .. " )" )
	local parent = getglobal( f ):GetParent():GetName()
	
	local cs = getglobal( parent .. "SelectedRow" ):GetText()
	local ns = tostring( getglobal( f ):GetID() )

	if ns == "0" then
		ARKINV:Print( "OOPS: widget [" .. f .. "] has no ID allocated" )
		return false
	end

	
	if cs ~= "-1" then
		getglobal( parent .. "Row" .. cs .. "Selected" ):Hide()
	end

	-- second click removes selection		
	if cs == ns then
		getglobal( parent .. "SelectedRow" ):SetText( "-1" )
		getglobal( parent .. "SelectedId" ):SetText( "-1" )
		return
	end

	getglobal( parent .. "SelectedRow" ):SetText( ns )
	getglobal( parent .. "SelectedId" ):SetText( getglobal( f .. "Id" ):GetText() )

	getglobal( f .. "Selected" ):Show()

end

function ARKINV:RuleTableReset( f )

	if not f then f = this:GetName() end
	-- hide and reset all rows
	
	local t = f .. "Table"
	
	local h = tonumber( getglobal( t .. "RowHeight" ):GetText() )
	local r = tonumber( getglobal( t .. "NumRows" ):GetText() )

	getglobal( t .. "SelectedRow" ):SetText( "-1" )
	for x = 1, r do
		getglobal( t .. "Row" .. x .. "Selected" ):Hide()
		getglobal( t .. "Row" .. x .. "Id" ):SetText( "-1" )
		getglobal( t .. "Row" .. x ):Hide()
		getglobal( t .. "Row" .. x ):SetHeight( h )
	end

end

function ARKINV:RuleTableRefresh( f )

	if not f then f = this:GetName() end
	
	local p = getglobal( f )
	if not p then
		ARKINV:Print( "Invalid value at ARKINV:RuleTableRefresh( [" .. ARKINV:MakeNullStringEmpty( f ) .. "] )" )
		return
	end

	ARKINV:Print( this:GetName() .. ", " .. f )
	
	f = f .. "View"

	if not f then
		f = this:GetParent():GetParent():GetName()
	end
	
	local ft = f .. "Table"
	local fs = f .. "Search"

	local height = tonumber( getglobal( ft .. "RowHeight" ):GetText() )
	local rows = tonumber( getglobal( ft .. "NumRows" ):GetText() )

	local line
	local lineplusoffset
	
	ARKINV:RuleTableReset( f )

	-- exit if no rules exist
	if not self.db.profile.option.global.rule then
		return
	end

	-- collect all valid items
	local filter = getglobal( fs .. "Filter"):GetText()
	--ARKINV:Print( "filter = [" .. filter .. "]" )

	local t = { }
	local tc = 0
	
	local ignore, iName, iQuality, iColor
	
	for k, d in pairs( self.db.profile.option.global.rule ) do

		ignore = false
			
		if filter ~= "" then
			if not string.find( string.lower( d.description ), string.lower( filter ) ) then
				ignore = true
			end
		end
		
		if not ignore then
			table.insert( t, { ["id"]=k, ["sorted"]=d.id, ["name"]=d.description, ["formula"]=d.formula } )
			tc = tc + 1
		end

	end

	if tc == 0 then
		return
	end
	
	-- sort them by name
	table.sort( t, function( a, b ) return a.sorted < b.sorted end )

	FauxScrollFrame_Update( getglobal( ft .. "Scroll" ), tc, rows, height )
	
	local linename, c, r
	
	for line = 1, rows do

		linename = ft .. "Row" .. line
		
		lineplusoffset = line + FauxScrollFrame_GetOffset( getglobal( ft .. "Scroll" ) )

		if lineplusoffset <= tc then

			c = ""
			r = t[lineplusoffset]
			
			getglobal( linename .. "Id" ):SetText( r.id )

			c = string.format( "%0.4f", r.sorted )
			getglobal( linename .. "C1" ):SetText( c )

			c = r.description
			if not c then c = "<no description set>" end
			getglobal( linename .. "C2" ):SetText( c )

			getglobal( linename .. "C3" ):SetText( r.formula )


			getglobal( linename ):Show()
			
			-- show selected if id is scrolled into view
			if getglobal( ft .. "SelectedId" ):GetText() == r.id then
				getglobal( ft .. "SelectedRow" ):SetText( line )
				getglobal( ft .. "Row" .. line .. "Selected" ):Show()
			end
			
		else
		
			getglobal( linename .. "Id"):SetText( "-1" )
			getglobal( linename ):Hide()
			
		end
	end

end

function ARKINV_RuleTableRefresh( f )
	ARKINV:RuleTableRefresh( f )
end
